﻿//BERNARDO DEL CASTILLO
//RAPTUS GAMES 2014
using UnityEngine;
using System.Collections;
public class ProceduralMesh : MonoBehaviour {

	MeshFilter filter;
	public float OuterRadius = 1.0f;
	public float InnerRadius = 0.3f;
	public int numSides = 50;
	// Use this for initialization
	void Start () {
	
	}
	
	// Update is called once per frame
	void Update () 
	{
		filter = gameObject.GetComponent<MeshFilter>();
		Mesh mesh = filter.mesh;
		mesh.Clear();
		
		int numVerticesDisk = numSides + numSides;
		#region Vertices
		
		// bottom + top + sides
		Vector3[] vertices = new Vector3[numVerticesDisk];
		int vert = 0;
		float twopi = Mathf.PI * 2f;
		
		while( vert < numSides )
		{
			float rad = (float)vert / numSides * twopi;
			RaycastHit hitinfo;
			Ray CurrentPt = new Ray(transform.position, new Vector3(Mathf.Cos(rad) * OuterRadius, 0f, Mathf.Sin(rad) * OuterRadius));
			Physics.Raycast(CurrentPt,out hitinfo, OuterRadius);
			
			if (hitinfo.collider != null)
			{
				vertices[vert] = hitinfo.point - transform.position;
			}
			else
			{
				vertices[vert] = CurrentPt.direction * OuterRadius;
			}
			vert++;
		}
		
		while (vert < numVerticesDisk)
		{

			if ( vertices[vert-numSides].sqrMagnitude < InnerRadius*InnerRadius)
			{
				vertices[vert] =  vertices[vert-numSides];
			}else
			{
				vertices[vert] =  vertices[vert-numSides].normalized * InnerRadius;
			}



			vert++;
		}
		
		
		
		#endregion
		
		#region Normals
		
		Vector3[] normals = new Vector3[vertices.Length];
		vert = 0;
		
		while( vert < vertices.Length )
		{
			normals[vert++] = Vector3.up;
		}
		
		#endregion
		
		#region UVs
		Vector2[] uvs = new Vector2[vertices.Length];
		
		// outer
		int u = 0;

		while (u < numSides*2)
		{
			float rad = Mathf.Atan2(vertices[u].z, vertices[u].x);
			
			float cosdist = (Mathf.Cos(rad));
			float sindist = (Mathf.Sin(rad));
			
			uvs[ u ] = new Vector2( cosdist* 0.5f, sindist*0.5f) *  vertices[u].magnitude/OuterRadius + Vector2.one*0.5f;
			u++;
		}
		
		#endregion 
		
		#region Triangles
		int numTriangles = numSides*2;
		int[] triangles = new int[numTriangles * 3];
		
		//RING
		int tri = 0;
		
		// 
		for ( int i = 0; tri< numTriangles; i++)
		{
			int j =  3 * tri;
			
			if ( tri == numTriangles-2)
			{
				triangles[ j ] = 0;
				triangles[ j+1 ] = numSides - 1;
				triangles[ j+2 ] = numSides*2 - 1;
				tri++;
				
				triangles[ j+3 ] = 0;
				triangles[ j+4 ] = numSides*2 - 1;
				triangles[ j+5 ] = numSides %  numTriangles;
				
				tri++;
			}
			else
			{
				triangles[ j ] = (i+1) % numTriangles;
				triangles[ j+1 ] = i % numTriangles;
				triangles[ j+2 ] = (numSides + i) %  numTriangles;
				tri++;
				
				triangles[ j+3 ] = (i+1) % numTriangles;
				triangles[ j+4 ] = (numSides + i) %  numTriangles;
				triangles[ j+5 ] = (numSides + i +1) %  numTriangles;
				
				tri++;
			}
			
		}
		#endregion
		
		mesh.vertices = vertices;
		mesh.normals = normals;
		mesh.uv = uvs;
		mesh.triangles = triangles;
		
		mesh.RecalculateBounds();
		mesh.Optimize();
	}
}
